// Gpib.cc

/*************************************************************************
* Copyright (c) 2003 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. mpfGpib is distributed
* subject to a Software License Agreement found in file LICENSE that
* is included with this distribution.
*************************************************************************/
// Author: Marty Kraimer
// Date: July 1998

#include <stdlib.h>
#include <stddef.h>
#include <string.h>
#include <stdio.h>
 
#include <semLib.h>
#include <intLib.h>
 
#include "WatchDog.h"
#include "DLList.h"
#include "Gpib.h"

static DLList *pGpibList = 0;

Gpib::Gpib()
  : ioStatus(gpibSuccess), ph(0), phParm(0), gpibName(0)
{
    ioComplete = semBCreate(SEM_Q_FIFO, SEM_EMPTY);
    pwg = new WatchDog;
}

int Gpib::registerName(const char *name)
{
    if(gpibName!=0) {
        printf("Gpib::registerName already registered %s\n",name);
        return(-1);
    }
    gpibName = name;
    if(pGpibList==0) pGpibList = new DLList;
    DLNode *node = DLList::allocNode();
    node->obj = this;
    pGpibList->addEnd(node);
    return(0);
}

Gpib *Gpib::find(const char *gpibName)
{
    if(pGpibList==0) pGpibList = new DLList;
    DLNode *node = pGpibList->getFirst();
    while(node) {
        Gpib *pGpib = (Gpib *)node->obj;
        if(::strcmp(gpibName,pGpib->gpibName)==0) {
            return(pGpib);
        }
        node = pGpibList->getNext(node);
    }
    return(0);
}

const char *Gpib::getGpibName()
{
    return(gpibName);
}

void Gpib::timoutHandler(void* v)
{
    Gpib* pGpib = (Gpib*)v;

    pGpib->ioStatus = gpibTimeout;
    semGive(pGpib->ioComplete);
}

void Gpib::startIoTimeout(double seconds)
{
    // Make sure there is no lingering semGive
    semTake(ioComplete,NO_WAIT);
    pwg->start(seconds,timoutHandler,this);
}
 
void Gpib::waitIo()
{
    semTake(ioComplete,WAIT_FOREVER);
    pwg->cancel();
}

void Gpib::waitIo(double seconds)
{
    pwg->start(seconds,timoutHandler,this);
    waitIo();
}

bool Gpib::setSniffHandler(
    const char *linkName,void* phParm,PEEK_HANDLER ph)
{
    Gpib *pGpib = find(linkName);
    if(!pGpib) {
        printf("%s link not found\n",linkName);
        return(false);
    }
    if(pGpib->ph!=0) {
        printf("%s already has a PEEK_HANDLER\n",linkName);
        return(0);
    }
    int lockKey = intLock();
    pGpib->ph = ph;
    pGpib->phParm = phParm;
    intUnlock(lockKey);
    return(true);
}

void Gpib::freeSniffHandler(const char *linkName)
{
    Gpib *pGpib = find(linkName);
    if(!pGpib) {
        printf("%s link not found\n",linkName);
        return;
    }
    int lockKey = intLock();
    pGpib->ph = 0;
    pGpib->phParm = 0;
    intUnlock(lockKey);
    return;
}

